@@ -44,9 +44,7 @@
                 terra::RGB(im_new) <- seq_len(3)
             } else if (inherits(img, "RemoteSpatialImage") || inherits(img, "StoredSpatialImage")) {
                 suppressWarnings(im_new <- rast(imgSource(img)))
-                if (packageVersion('terra') == as.package_version("1.7.83") ||
-                    packageVersion('terra') >= as.package_version("1.8.10"))
-                    im_new <- terra::flip(im_new)
+                if (.terra_flip()) im_new <- terra::flip(im_new)
             } else {
                 warning("Don't know how to convert image ", i, " to SpatRaster, ",
                         "dropping image.")
@@ -119,7 +117,7 @@
 #' )
 #' ## using spacerangerOut folder
 #' tv <- TENxVisium(
-#'     spacerangerOut = sample_dir, processing = "raw", images = "lowres"
+#'     spacerangerOut = file.path(sample_dir, "outs"), processing = "raw", images = "lowres"
 #' )
 #' spe <- import(tv)
 #' # There can't be duplicate barcodes
