@@ -12,32 +12,29 @@
 #' @inheritParams SpatialFeatureExperiment
 #' @inheritParams DropletUtils::read10xCounts
 #' @inheritParams SpatialExperiment::read10xVisium
-#' @param sample To be consistent with \code{SpatialExperiment::read10xVisium},
-#'   one or more directories with Space Ranger output for a Visium sample. It is
-#'   assumed to have the \code{outs} directory in it but this can be overridden
-#'   with the \code{dirs} argument.
+#' @param sample Deprecated, use the \code{dirs} argument instead.
 #' @param type Either "HDF5", and the matrix will be represented as
 #'   \code{TENxMatrix}, or "sparse", and the matrix will be read as a
 #'   \code{dgCMatrix}.
 #' @param dirs Directory for each sample that contains the \code{spatial} and
-#'   \code{raw/filtered_featues_bc_matrix} directories. By default, the
-#'   \code{outs} directory under the directory specified in the \code{samples}
-#'   argument, as in Space Ranger output. Change the \code{dirs} argument if you
-#'   have moved or renamed the output directory.
+#'   \code{raw/filtered_featues_bc_matrix} directories.
 #' @param unit Whether to use pixels in full resolution image or microns as the
 #'   unit. If using microns, then spacing between spots in pixels will be used
 #'   to convert the coordinates into microns, as the spacing is known to be 100
 #'   microns. This is used to plot scale bar.
-#' @param load Deprecated. Not used, kept for backward compatibility for now.
 #' @param flip Whether to flip the geometries or the images, because in
 #'   \code{sf} and \code{terra}, the geometries use the Cartesian coordinates
 #'   greater y coordinates going up, while in images, greater y values go down.
 #'   Originally the Visium spots are in pixels in full res image. Either the
 #'   image or the geometry needs to be flipped for them match in the Cartesian
 #'   coordinate system.
+#' @param read_spatial_enrichment Logical, whether to read the
+#'   `spatial_enrichment.csv` file from Visium output if the file is present and
+#'   add its contents to `rowData`.
 #' @importFrom rjson fromJSON
 #' @importFrom SummarizedExperiment rowData<-
 #' @importFrom utils read.csv
+#' @importFrom SpatialExperiment spatialCoordsNames<-
 #' @concept Read data into SFE
 #' @importFrom DropletUtils read10xCounts
 #' @note It is assumed that the images have not been cropped. Otherwise the
@@ -50,54 +47,56 @@
 #' dir <- system.file("extdata", package = "SpatialFeatureExperiment")
 #'
 #' sample_ids <- c("sample01", "sample02")
-#' samples <- file.path(dir, sample_ids)
+#' samples <- file.path(dir, sample_ids, "outs")
 #'
 #' list.files(samples[1])
 #' list.files(file.path(samples[1], "spatial"))
-#' (sfe <- read10xVisiumSFE(samples, sample_id = sample_ids,
-#'     type = "sparse", data = "filtered",
-#'     load = FALSE
+#' (sfe <- read10xVisiumSFE(dirs = samples, sample_id = sample_ids,
+#'     type = "sparse", data = "filtered"
 #' ))
 #' 
-read10xVisiumSFE <- function(samples = "",
-                             dirs = file.path(samples, "outs"),
+read10xVisiumSFE <- function(samples = deprecated(),
+                             dirs = NULL,
                              sample_id = paste0(
                                  "sample",
                                  sprintf(
                                      "%02d",
-                                     seq_along(samples)
+                                     seq_along(dirs)
                                  )
                              ),
                              type = c("HDF5", "sparse"),
                              data = c("filtered", "raw"),
                              images = c("lowres", "hires"),
                              unit = c("full_res_image_pixel", "micron"),
-                             style = "W", zero.policy = NULL, load = deprecated(),
+                             style = "W", zero.policy = NULL,
                              row.names = c("id", "symbol"),
-                             flip = c("geometry", "image", "none")) {
+                             flip = c("geometry", "image", "none"),
+                             read_spatial_enrichment = TRUE) {
+    check_installed("DropletUtils")
+    if (is_present(samples)) {
+        deprecate_warn("1.12.0", "read10xVisiumSFE(samples)",
+                       "read10xVisiuimSFE(dirs)")
+        dirs <- dirs %||% file.path(samples, "outs")
+    }
     type <- match.arg(type)
     data <- match.arg(data)
     unit <- match.arg(unit)
     flip <- match.arg(flip)
-    if (packageVersion('terra') >= as.package_version("1.7.83"))
-        flip <- "none"
-    if (is_present(load)) {
-        deprecate_warn("1.8.0", "read10xVisiumSFE(load)")
-    }
+    if (.terra_flip()) flip <- "none"
     images <- match.arg(images, several.ok = TRUE)
     row.names <- match.arg(row.names)
     enrichment_feature <- switch(row.names,
                                  id = "Feature.ID",
                                  symbol = "Feature.Name")
     # Read one sample at a time, in order to get spot diameter one sample at a time
-    sfes <- lapply(seq_along(samples), function(i) {
+    sfes <- lapply(seq_along(dirs), function(i) {
         o <- .read10xVisium(dirs[i], sample_id[i],
                             type, data, images, 
                             row.names  = row.names, flip = flip, VisiumHD = FALSE,
                             unit = unit, zero.policy = zero.policy, style = style)
         # Add spatial enrichment if present
         fn <- file.path(dirs[i], "spatial", "spatial_enrichment.csv")
-        if (file.exists(fn)) {
+        if (file.exists(fn) && read_spatial_enrichment) {
             enrichment <- read.csv(fn)
             row_inds <- match(rownames(o), enrichment[[enrichment_feature]])
             # Let me not worry about different samples having different genes for now
@@ -138,21 +137,18 @@ read10xVisiumSFE <- function(samples = "",
            sample_id = "sample01",
            type = c("HDF5", "sparse"),
            data = c("filtered", "raw"),
-           images = "lowres",
+           images = c("lowres", "hires"),
            row.names = "id",
            VisiumHD = FALSE, flip = c("geometry", "image"),
            add_graph = TRUE, unit = c("full_res_image_pixel", "micron"),
            rotate_hd = FALSE, zero.policy = TRUE, style = "W",
            add_centroids = FALSE) {
-    if (!requireNamespace("DropletUtils", quietly = TRUE)) {
-      warning("DropletUtils package must be installed to use read10xVisium()")
-    }
     type <- match.arg(type)
     data <- match.arg(data)
     imgs <- c("lowres", "hires", "detected", "aligned")
     imgs <- match.arg(images, imgs, several.ok = TRUE)
     if (!VisiumHD) {
-        if (basename(sample) != "outs" && "outs" %in% list.files(sample))
+        if (basename(sample) != "outs" && "outs" %in% list.files(sample, include.dirs = TRUE))
             sample <- file.path(sample, "outs")
     }
     message(paste0(">>> 10X ", ifelse(VisiumHD, "VisiumHD", "Visium"),
@@ -326,8 +322,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
     data <- match.arg(data)
     unit <- match.arg(unit)
     flip <- match.arg(flip)
-    if (packageVersion('terra') >= as.package_version("1.7.83"))
-        flip <- "none"
+    if (.terra_flip()) flip <- "none"
     images <- match.arg(images, several.ok = TRUE)
     bin_size <- match.arg(as.character(bin_size), choices = c("2", "8", "16"),
                           several.ok = TRUE) |> 
@@ -340,7 +335,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
         grep("binned_out|square|um$", dirs_check) |> any()
     if (sanity_passed) {
         # match sample names with bin_size
-        samples <- grep(paste0(bin_size, collapse = "|"), dirs_check, value = TRUE)
+        samples <- grep(paste0(paste0(bin_size, collapse = "|"), "um$"), dirs_check, value = TRUE)
     } else {
         stop("data_dir should contain or be `square_0xxum`")
     }
@@ -416,6 +411,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
 #' @importFrom sf st_is_empty st_drop_geometry
 #' @importFrom BiocParallel bplapply
 #' @importFrom utils head
+#' @importFrom data.table :=
 .filter_polygons <- function(polys, min_area,
                              is_Xenium = FALSE, # indicate if input tech is Xenium or not
                              BPPARAM = SerialParam()) {
@@ -434,7 +430,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
                 " cells with (nested) polygon lists", "\n",
                 ">>> ..applying filtering") }
     # add sequence of numbers as temporary column
-    polys$ID_row <- seq_len(length.out = nrow(polys))
+    polys$ID_row <- as.character(seq_len(length.out = nrow(polys)))
     polys.ID_row <- polys$ID_row
     # remove empty elements
     polys <- polys[!st_is_empty(polys), ]
@@ -468,7 +464,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
             inds <- which(areas > min_area)
             polys <- polys[inds, ]
         }
-        dupl_inds <- which(polys[[cell_ID]] |> duplicated())
+        dupl_inds <- which(polys$ID_row |> duplicated())
         # filter polygons with multiple pieces in single cell segmentation
         if (length(dupl_inds)) {
             warning("There are ", length(dupl_inds), " cells with multiple", " pieces in cell segmentation", 
@@ -477,26 +473,23 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
                     paste(dupl_inds |> head(10),
                           collapse = ", "),
                     ". The largest piece is kept.")
-            dupl_cells <- polys[[cell_ID]][dupl_inds]
+            dupl_cells <- polys$ID_row[dupl_inds]
             # areas of polygons with multiple pieces
             dupl_areas <- areas[which(names(areas) %in% polys$ID_row[dupl_inds])]
             if (!is.null(min_area))
                 # filter with minimal area
                 dupl_areas <- dupl_areas[dupl_areas > min_area]
             # get clean polygons
-            add_geo <-
-                # this can take time if not parallelized and many artifacts to be removed
-                bplapply(seq_along(names(dupl_areas) |> unique()), 
-                         function(i) {
-                             which_keep <- 
-                                 dupl_areas[names(dupl_areas) %in% unique(names(dupl_areas))[i]] |> 
-                                 which.max()
-                             polys[polys[[cell_ID]] %in% dupl_cells[i], ] |> 
-                                 st_geometry() |> _[[which_keep]]
-                         }, BPPARAM = BPPARAM) |> st_sfc()
+            dupl_ids <- unique(names(dupl_areas))
+            dupl_area_dt <- data.table::data.table(area = dupl_areas, name = names(dupl_areas))
+            poly_df <- polys[polys$ID_row %in% unique(names(dupl_areas)),]
+            stopifnot("poly_df and dupl_areas must have identical sorting" = all(poly_df$ID_row == dupl_area_dt$name))
+            # ^ Will this always hold? TODO sort dupl_area_dt accordingly if not
+            id_max_dt <- dupl_area_dt[,gmax := max(area), by = name][, gi := seq_len(.N), by = name][, i := .I][area == gmax]
+            add_geo <- poly_df[id_max_dt$i,] |> st_geometry() |> st_sfc()
             # add clean geometries
             polys_add <- 
-                polys[polys[[cell_ID]] %in% dupl_cells, ] |> 
+                polys[polys$ID_row %in% dupl_cells, ] |> 
                 st_drop_geometry() |>
                 dplyr::distinct(!!rlang::sym(cell_ID),
                                 .keep_all = TRUE)
@@ -505,7 +498,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
             colnames(polys_add) <- colnames(polys)
             polys <- 
                 # data.table is faster than rbind or dplyr::bind_rows
-                data.table::rbindlist(list(polys[!polys[[cell_ID]] %in% dupl_cells,], 
+                data.table::rbindlist(list(polys[!polys$ID_row %in% dupl_cells,], 
                                            polys_add)) |> 
                 as.data.frame() |> st_as_sf()
             # sort by ID_row
@@ -538,15 +531,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
 }
 
 .if_flip_img <- function(fn, max_flip) {
-    max_flip <- toupper(max_flip)
-    unit <- gsub("^[0-9.]+\\s?", "", max_flip)
-    if (!unit %in% c("MB", "GB"))
-        stop("max_flip must be in either MB or GB.")
-    max_flip <- as.numeric(gsub("\\s?[A-Z.]+$", "", max_flip))
-    max_flip <- switch (unit,
-                        MB = max_flip * 1024^2,
-                        GB = max_flip * 1024^3
-    )
+    max_flip <- .size_str2num(max_flip)
     size <- file.info(fn)[["size"]] # NA if file doesn't exist
     size < max_flip
 }
@@ -652,7 +637,7 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
 #' The coordinates are in microns. Cell centroids are read into
 #' \code{\link{colGeometry}} "centroids", and cell segmentations are read into
 #' \code{colGeometry} "cellSeg". The image(s) (polyT, DAPI, and cell boundaries)
-#' are also read as \code{\link{SpatRaster}} objects so they are not loaded into
+#' are also read as \code{\link[terra]{SpatRaster}} objects so they are not loaded into
 #' memory unless necessary. Because the image's origin is the top left while the
 #' geometry's origin is bottom left, either the image or the geometry needs to
 #' be flipped. Because the image accompanying MERFISH datasets are usually very
@@ -689,11 +674,11 @@ readVisiumHD <- function(data_dir, bin_size = c(2L, 8L, 16L),
 #' @param use_cellpose Whether to read the parquet files from CellPose cell
 #'   segmentation. If \code{FALSE}, cell segmentation will be read from the HDF5
 #'   files. Note that reading HDF5 files for numerous FOVs is very slow.
-#' @param BPPARAM A \code{\link{BiocParallelParam}} object specifying parallel
+#' @param BPPARAM A \code{\link[BiocParallel]{BiocParallelParam}} object specifying parallel
 #'   processing backend and number of threads to use for parallelizable tasks:
 #'   \enumerate{ \item To load cell segmentation from HDF5 files from different
 #'   fields of view (FOVs) with multiple cores. A progress bar can be configured
-#'   in the \code{\link{BiocParallelParam}} object. When there are numerous
+#'   in the \code{\link[BiocParallel]{BiocParallelParam}} object. When there are numerous
 #'   FOVs, reading in the geometries can be time consuming, so we recommend
 #'   using a server and larger number of threads. This argument is not used if
 #'   \code{use_cellpose = TRUE} and the parquet file is present.
@@ -747,8 +732,7 @@ readVizgen <- function(data_dir,
     check_installed("sfarrow")
     data_dir <- normalizePath(data_dir, mustWork = TRUE)
     flip <- match.arg(flip)
-    if (packageVersion('terra') >= as.package_version("1.7.83"))
-        flip <- "none"
+    if (.terra_flip()) flip <- "none"
     image <- match.arg(image, several.ok = TRUE)
     if ((any(z < 0) || any(z > 6)) && z != "all") {
         stop("z must be beween 0 and 6 (inclusive).")
@@ -955,7 +939,8 @@ readVizgen <- function(data_dir,
 #' Read CosMX data into SFE
 #'
 #' This function reads the standard CosMX output into an SFE object, as in
-#' "Basic Data Files" on the Nanostring website.
+#' "Basic Data Files" on the Nanostring website. For new version of CosMX, these
+#' files are the flat files in the AtoMX output.
 #'
 #' @inheritParams readVizgen
 #' @param z Integer z index or "all" to indicate which z-planes to read for the
@@ -986,13 +971,13 @@ readCosMX <- function(data_dir,
                       z_option = c("3d", "split")) {
     check_installed("sfarrow")
     data_dir <- normalizePath(data_dir, mustWork = TRUE)
-    fns <- list.files(data_dir, pattern = "\\.csv$", full.names = TRUE)
-    fn_metadata <- grep("metadata", fns, value = TRUE)
-    fn_mat <- grep("exprMat", fns, value = TRUE)
-    fn_polys <- grep("polygons", fns, value = TRUE)
+    fns <- list.files(data_dir, pattern = "\\.csv.*$", full.names = TRUE)
+    fn_metadata <- grep("metadata", fns, value = TRUE)[1]
+    fn_mat <- grep("exprMat", fns, value = TRUE)[1]
+    fn_polys <- grep("polygons", fns, value = TRUE)[1]
 
     meta <- fread(fn_metadata)
-    mat <- fread(fn_mat) # TODO: write to h5 or mtx. Consult alabaster.sce
+    mat <- fread(fn_mat)
     
 
     meta$cell_ID <- paste(meta$cell_ID, meta$fov, sep = "_")
@@ -1024,6 +1009,7 @@ readCosMX <- function(data_dir,
     }
 
     sfe <- SpatialFeatureExperiment(list(counts = mat), colData = meta,
+                                    sample_id = sample_id,
                                     spatialCoordsNames = c("CenterX_global_px", "CenterY_global_px"),
                                     unit = "full_res_image_pixel")
     # sanity on geometries
@@ -1044,7 +1030,7 @@ readCosMX <- function(data_dir,
 # helper function to convert from raw bytes to character
 .rawToChar_df <- function(input_df, BPPARAM = SerialParam()) {
     convert_ids <-
-        lapply(input_df, function(x) is(x, "arrow_binary")) |> unlist() |> which()
+        lapply(input_df, function(x) inherits(x, "arrow_binary")) |> unlist() |> which()
     if (any(convert_ids)) {
         message(">>> Converting columns with raw bytes (ie 'arrow_binary') to character")
         cols_converted <-
@@ -1058,7 +1044,7 @@ readCosMX <- function(data_dir,
             input_df[,convert_ids][[i]] <- unlist(cols_converted[[i]])
         }
     }
-    if (!is(input_df, "data.table")) {
+    if (!inherits(input_df, "data.table")) {
         input_df <- data.table::as.data.table(input_df)
     }
     return(input_df)
