@@ -384,3 +384,11 @@ getTechTxFields <- function(tech, data_dir = NULL) {
     bb_area <- bbox |> st_bbox() |> st_as_sfc() |> st_area()
     bb_area/tot_area
 }
+
+.terra_flip <- function() {
+    suppressWarnings(img <- rast(system.file(file.path("extdata", "sample01", 
+                                                       "outs", "spatial", 
+                                                       "tissue_lowres_image.png"),
+                                             package = "SpatialFeatureExperiment")))
+    as.vector(img[18,4,2] > 200)
+}
